@@ -27,7 +27,7 @@
 
 #if (defined ALLEGRO_LINUX_FBCON) && ((!defined ALLEGRO_WITH_MODULES) || (defined ALLEGRO_MODULE))
 
-#if !defined(_POSIX_MAPPED_FILES) || !defined(HAVE_MMAP)
+#if !defined(_POSIX_MAPPED_FILES) || !defined(ALLEGRO_HAVE_MMAP)
 #error "Sorry, mapped files are required for Linux console Allegro to work!"
 #endif
 
@@ -48,6 +48,14 @@
 #define FB_VBLANK_STICKY 0
 #endif
 
+
+
+#define PREFIX_I                "al-fbcon INFO: "
+#define PREFIX_W                "al-fbcon WARNING: "
+#define PREFIX_E                "al-fbcon ERROR: "
+
+
+
 static BITMAP *fb_init(int w, int h, int v_w, int v_h, int color_depth);
 static int fb_open_device(void);
 static void fb_exit(BITMAP *b);
@@ -79,6 +87,7 @@ GFX_DRIVER gfx_fbcon =
    NULL,                         /* no drawing mode hook */
    fb_save,
    fb_restore,
+   NULL,    // AL_METHOD(void, set_blender_mode, (int mode, int r, int g, int b, int a));
    NULL,                         /* no fetch mode hook */
    0, 0,
    TRUE,
@@ -88,6 +97,8 @@ GFX_DRIVER gfx_fbcon =
 
 static char fb_desc[256] = EMPTY_STRING;     /* description string */
 
+static int fb_mode_read = FALSE;             /* has orig_mode been read? */
+
 static struct fb_fix_screeninfo fix_info;    /* read-only video mode info */
 static struct fb_var_screeninfo orig_mode;   /* original video mode info */
 static struct fb_var_screeninfo my_mode;     /* my video mode info */
@@ -102,12 +113,110 @@ static int fb_approx;                        /* emulate small resolution */
    static int vblank_flags;                  /* supports retrace detection? */
 #endif
 
+static int in_fb_restore;
+
 
 
 static int update_timings(struct fb_var_screeninfo *mode);
 
 
 
+static void set_ramp_cmap(AL_CONST struct fb_fix_screeninfo *fix,
+                          AL_CONST struct fb_var_screeninfo *var)
+{
+   struct fb_cmap cmap;
+   static unsigned short r[256], g[256], b[256]; /* 1.5 KB, so in .bss */
+   int rlen, glen, blen, rdiv, gdiv, bdiv;
+   unsigned int c;
+
+   ASSERT(fix);
+   ASSERT(var);
+   ASSERT(fix->visual == FB_VISUAL_DIRECTCOLOR);
+   
+   /* initialize what is common */
+   cmap.start = 0;
+   cmap.red = r;
+   cmap.green = g;
+   cmap.blue = b;
+   cmap.transp = NULL;
+
+   /* from the mode's color depth information */
+   rlen = 1<<var->red.length;
+   glen = 1<<var->green.length;
+   blen = 1<<var->blue.length;
+   cmap.len = MAX(rlen, MAX(glen, blen));
+   ASSERT(cmap.len <= 256);
+   if (cmap.len > 256)
+      cmap.len = 256; /* are there cards with more than 8 bits per color ? */
+
+   rdiv = rlen<=1 ? 1 : rlen-1;
+   gdiv = glen<=1 ? 1 : glen-1;
+   bdiv = blen<=1 ? 1 : blen-1;
+  
+   /* the easy case (should not happen) */
+   if (cmap.len == 0)
+      return;
+
+   /* build the colormap */
+   for (c=0; c<cmap.len; ++c) {
+      cmap.red[c] = c*65535/rdiv;
+      cmap.green[c] = c*65535/gdiv;
+      cmap.blue[c] = c*65535/bdiv;
+   }
+
+   /* wait a little to set colors once the frame is drawn */
+   fb_vsync();
+
+   /* ioctl to the FB driver with our colormap */
+   if (ioctl(fbfd, FBIOPUTCMAP, &cmap)) {
+      /* well, we can continue with potentially screwed up colors, I guess */
+      ASSERT(0);
+   }
+}
+
+
+
+static void calculate_refresh_rate(AL_CONST struct fb_var_screeninfo *mode)
+{
+   int h, v, hz;
+
+   ASSERT(mode);
+
+   h = mode->left_margin+mode->xres+mode->left_margin+mode->hsync_len;
+   v = mode->upper_margin+mode->yres+mode->lower_margin+mode->vsync_len;
+   hz = (int)(0.5f+1.0e12f/((float)mode->pixclock*h*v));
+   _set_current_refresh_rate(hz);
+}
+
+
+
+/* __al_linux_get_fb_color_depth:
+ *  Get the colour depth of the framebuffer
+ */
+int __al_linux_get_fb_color_depth(void)
+{
+   if ((!fb_mode_read) && (fb_open_device() != 0))
+      return 0;
+   return orig_mode.bits_per_pixel;
+}
+
+
+
+/* __al_linux_get_fb_resolution:
+ *  Get the resolution of the framebuffer
+ */
+int __al_linux_get_fb_resolution(int *width, int *height)
+{
+   if ((!fb_mode_read) && (fb_open_device() != 0))
+      return -1;
+
+   *width = orig_mode.xres;
+   *height = orig_mode.yres;
+   return 0;
+}
+
+
+
 /* fb_init:
  *  Sets a graphics mode.
  */
@@ -119,7 +228,7 @@ static BITMAP *fb_init(int w, int h, int v_w, int v_h, int color_depth)
    char tmp[16];
 
    /* open framebuffer and store info in global variables */
-   if (fb_open_device() != 0)
+   if ((!fb_mode_read) && (fb_open_device() != 0))
       return NULL;
 
    /* look for a nice graphics mode in several passes */
@@ -132,12 +241,21 @@ static BITMAP *fb_init(int w, int h, int v_w, int v_h, int color_depth)
    if (((!w) && (!h)) || _safe_gfx_mode_change) {
       w = orig_mode.xres;
       h = orig_mode.yres;
+      TRACE(PREFIX_I "User didn't ask for a resolution, and we are trying "
+	    "to set a safe mode, so I will try resolution %dx%d\n", w, h);
    }
 
    if (_safe_gfx_mode_change) tries = -1;
    else tries = 0;
-   
+
+   if (__al_linux_console_graphics() != 0) {
+      ioctl(fbfd, FBIOPUT_VSCREENINFO, &orig_mode);
+      close(fbfd);
+      return NULL;
+   }
+
    for (; tries<3; tries++) {
+      TRACE(PREFIX_I "...try number %d...\n", tries);
       my_mode = orig_mode;
 
       switch (tries) {
@@ -180,6 +298,7 @@ static BITMAP *fb_init(int w, int h, int v_w, int v_h, int color_depth)
 
       my_mode.bits_per_pixel = color_depth;
       my_mode.grayscale = 0;
+      my_mode.activate = FB_ACTIVATE_NOW;
       my_mode.xoffset = 0;
       my_mode.yoffset = 0;
 
@@ -241,27 +360,45 @@ static BITMAP *fb_init(int w, int h, int v_w, int v_h, int color_depth)
 	 continue;
 
       /* try to set the mode */
-      if (ioctl(fbfd, FBIOPUT_VSCREENINFO, &my_mode) == 0)
-	 goto got_a_nice_mode;
+      if (ioctl(fbfd, FBIOPUT_VSCREENINFO, &my_mode) == 0) {
+	 if (my_mode.bits_per_pixel == (unsigned)color_depth)
+	    goto got_a_nice_mode;
+      }
    }
 
    /* oops! */
-   if (_safe_gfx_mode_change)
+   if (_safe_gfx_mode_change) {
       set_color_depth(original_color_depth);
+      TRACE(PREFIX_I "Restoring color depth %d\n", original_color_depth);
+   }
       
    ioctl(fbfd, FBIOPUT_VSCREENINFO, &orig_mode);
+   __al_linux_console_text();
    close(fbfd);
    ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Framebuffer resolution not available"));
+   TRACE(PREFIX_E "Resolution %dx%d not available...\n", w, h);
    return NULL;
 
    got_a_nice_mode:
 
+   /* get the fb_fix_screeninfo again, as it may have changed */
+   if (ioctl(fbfd, FBIOGET_FSCREENINFO, &fix_info) != 0) {
+      ioctl(fbfd, FBIOPUT_VSCREENINFO, &orig_mode);
+      __al_linux_console_text();
+      close(fbfd);
+      ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Framebuffer ioctl() failed"));
+      return NULL;
+   }
+
    /* map the framebuffer */
    fbaddr = mmap(NULL, fix_info.smem_len, PROT_READ | PROT_WRITE, MAP_SHARED, fbfd, 0);
    if (fbaddr == MAP_FAILED) {
       ioctl(fbfd, FBIOPUT_VSCREENINFO, &orig_mode);
+      __al_linux_console_text();
       close(fbfd);
       ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Can't map framebuffer"));
+      TRACE(PREFIX_E "Couldn't map framebuffer for %dx%d. Restored old "
+	    "resolution.\n", w, h);
       return NULL;
    }
 
@@ -271,7 +408,7 @@ static BITMAP *fb_init(int w, int h, int v_w, int v_h, int color_depth)
 
    gfx_fbcon.vid_mem = fix_info.smem_len;
 
-   stride = my_mode.xres_virtual * BYTES_PER_PIXEL(color_depth);
+   stride = fix_info.line_length;
    v_w = my_mode.xres_virtual;
    v_h = my_mode.yres_virtual;
    p = fbaddr;
@@ -288,7 +425,9 @@ static BITMAP *fb_init(int w, int h, int v_w, int v_h, int color_depth)
    if (!b) {
       ioctl(fbfd, FBIOPUT_VSCREENINFO, &orig_mode);
       munmap(fbaddr, fix_info.smem_len);
+      __al_linux_console_text();
       close(fbfd);
+      TRACE(PREFIX_E "Couldn't make bitmap `b', sorry.\n");
       return NULL;
    }
 
@@ -378,12 +517,22 @@ static BITMAP *fb_init(int w, int h, int v_w, int v_h, int color_depth)
    else
       gfx_fbcon.scroll = NULL;
 
-   __al_linux_console_graphics();
-
    if (fb_approx)
       memset(fbaddr, 0, gfx_fbcon.vid_mem);
 
    fb_save_cmap();    /* Maybe we should fill in our default palette too... */
+
+   /* for directcolor modes, set up a ramp colormap, so colors are mapped
+      onto themselves, so to speak */
+   if (fix_info.visual == FB_VISUAL_DIRECTCOLOR)
+      set_ramp_cmap(&fix_info, &my_mode);
+
+   calculate_refresh_rate(&my_mode);
+  
+   /* Note: Locked in __al_linux_console_graphics(). */ 
+   __al_linux_display_switch_lock(FALSE, FALSE);
+ 
+   TRACE(PREFIX_I "Got a bitmap %dx%dx%d\n", b->w, b->h, bitmap_color_depth(b));
    return b;
 }
 
@@ -407,16 +556,16 @@ static int fb_open_device(void)
       p = getenv("FRAMEBUFFER");
 
       if ((p) && (p[0])) {
-	 strncpy(fname, p, sizeof(fname)-1);
-	 fname[sizeof(fname)-1] = 0;
+	 _al_sane_strncpy(fname, p, sizeof(fname));
       }
       else
-	 strcpy(fname, "/dev/fb0");
+	 _al_sane_strncpy(fname, "/dev/fb0", 1024);
    }
 
    /* open the framebuffer device */
    if ((fbfd = open(fname, O_RDWR)) < 0) {
       uszprintf(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Can't open framebuffer %s"), uconvert_ascii(fname, tmp1));
+      TRACE(PREFIX_E "Couldn't open %s\n", fname);
       return 1;
    }
 
@@ -424,9 +573,12 @@ static int fb_open_device(void)
    if ((ioctl(fbfd, FBIOGET_FSCREENINFO, &fix_info) != 0) ||
        (ioctl(fbfd, FBIOGET_VSCREENINFO, &orig_mode) != 0)) {
       ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Framebuffer ioctl() failed"));
+      TRACE(PREFIX_E "ioctl() failed\n");
       return 2;
    }
 
+   TRACE(PREFIX_I "fb device %s opened successfully.\n", fname);
+   fb_mode_read = TRUE;
    return 0;
 }
 
@@ -437,6 +589,11 @@ static int fb_open_device(void)
  */
 static void fb_exit(BITMAP *b)
 {
+   TRACE(PREFIX_I "Unsetting video mode.\n");
+  
+   /* Note: Unlocked in __al_linux_console_text(). */ 
+   __al_linux_display_switch_lock(TRUE, TRUE);
+
    ioctl(fbfd, FBIOPUT_VSCREENINFO, &orig_mode);
    fb_restore_cmap();
 
@@ -444,6 +601,7 @@ static void fb_exit(BITMAP *b)
    close(fbfd);
 
    __al_linux_console_text();
+   fb_mode_read = FALSE;
 }
 
 
@@ -451,7 +609,7 @@ static void fb_exit(BITMAP *b)
 /* fb_save:
  *  Saves the graphics state.
  */
-static void fb_save()
+static void fb_save(void)
 {
    ioctl(fbfd, FBIOPUT_VSCREENINFO, &orig_mode);
 }
@@ -461,12 +619,21 @@ static void fb_save()
 /* fb_restore:
  *  Restores the graphics state.
  */
-static void fb_restore()
+static void fb_restore(void)
 {
+   ASSERT(!in_fb_restore);
+
+   in_fb_restore = TRUE;
+   
    ioctl(fbfd, FBIOPUT_VSCREENINFO, &my_mode);
 
    if (fb_approx)
       memset(fbaddr, 0, gfx_fbcon.vid_mem);
+
+   if (fix_info.visual == FB_VISUAL_DIRECTCOLOR)
+      set_ramp_cmap(&fix_info, &my_mode);
+
+   in_fb_restore = FALSE;
 }
 
 
@@ -483,13 +650,29 @@ static int fb_scroll(int x, int y)
 
    ret = ioctl(fbfd, FBIOPAN_DISPLAY, &my_mode);
 
-   fb_vsync();
-
- #ifdef FBIOGET_VBLANK
-   if (!vblank_flags)
- #endif
+   if (_wait_for_vsync) {
+      /* On a lot of graphics hardware the scroll registers don't
+       * take effect until the start of the next frame, so when you do a
+       * scroll you want to make sure you've waited at least until then.
+       * If you have reliable vsyncing, one call to vysnc is enough,
+       * but if you don't then the first call waits some time between 0
+       * and 1/60th of a second, and since it's not really synced to the
+       * retrace, this may or may not include an actual retrace. However,
+       * by making a second call, we then wait for a full 1/60th of a second
+       * after the first call, and this is sure to include a retrace,
+       * assuming the refresh rate is at least 60Hz. Sometimes overall
+       * we'll have waited past two retraces, or even more if the refresh
+       * rate is much higher than the fake vblank timer frequency, but
+       * it's still better than sometimes missing the retrace completely.
+       */
       fb_vsync();
 
+    #ifdef FBIOGET_VBLANK
+      if (!vblank_flags)
+    #endif
+         fb_vsync();
+   }
+
    return (ret) ? -1 : 0;
 }
 
@@ -544,7 +727,11 @@ static void fb_vsync(void)
  #endif
 
    /* bodged implementation for when the framebuffer doesn't support it */
-   if (_timer_installed) {
+   /* We must not to run this loop while returning from a VT switch as timer
+    * "interrupts" will not be running at that time so retrace_count will
+    * remain constant.
+    */
+   if (_timer_installed && !in_fb_restore) {
       prev = retrace_count;
 
       do {
@@ -562,6 +749,7 @@ static void fb_set_palette(AL_CONST RGB *p, int from, int to, int vsync)
    unsigned short r[256], g[256], b[256];
    struct fb_cmap cmap;
    int i;
+   ASSERT(p);
 
    cmap.start = from;
    cmap.len = to-from+1;
@@ -607,8 +795,8 @@ static void fb_do_cmap (int ioctlno)
  */
 static void fb_save_cmap (void)
 {
-	if (orig_cmap_data) free (orig_cmap_data);   /* can't happen */
-	orig_cmap_data = malloc (sizeof *orig_cmap_data* 768);
+	if (orig_cmap_data) _AL_FREE (orig_cmap_data);   /* can't happen */
+	orig_cmap_data = _AL_MALLOC_ATOMIC (sizeof *orig_cmap_data* 768);
 	if (orig_cmap_data)
 		fb_do_cmap (FBIOGETCMAP);
 }
@@ -619,7 +807,7 @@ static void fb_restore_cmap (void)
 {
 	if (orig_cmap_data) {
 		fb_do_cmap (FBIOPUTCMAP);
-		free (orig_cmap_data);
+		_AL_FREE (orig_cmap_data);
 		orig_cmap_data = NULL;
 	}
 }
@@ -719,8 +907,8 @@ static char *get_line (FILE *file)
 {
    static char buffer[1024];
    char *ch;
-
    ASSERT (file);
+
    if (!fgets (buffer, sizeof (buffer), file))
       return 0;
 
@@ -770,9 +958,9 @@ static int read_fbmodes_file (int w, int h)
       }
 
       if (!strcmp (s, "mode")) {
-	 free (mode_id);
-	 free (geometry);
-	 free (timings);
+	 _AL_FREE (mode_id);
+	 _AL_FREE (geometry);
+	 _AL_FREE (timings);
 	 mode_id = strdup (t);
 	 geometry = timings = NULL;
 	 sync = 0;
@@ -799,15 +987,15 @@ static int read_fbmodes_file (int w, int h)
 	       s = NULL;
 	    }
 	 }
-	 free (mode_id);
-	 free (geometry);
-	 free (timings);
+	 _AL_FREE (mode_id);
+	 _AL_FREE (geometry);
+	 _AL_FREE (timings);
 	 mode_id = geometry = timings = NULL;
       } else if (!strcmp (s, "geometry")) {
-	 free (geometry);
+	 _AL_FREE (geometry);
 	 geometry = strdup (t);
       } else if (!strcmp (s, "timings")) {
-	 free (timings);
+	 _AL_FREE (timings);
 	 timings = strdup (t);
       } else if (!strcmp (s, "hsync")) {
 	 #define set_bit(var,bit,on) var = ((var) &~ (bit)) | ((on) ? bit : 0)
@@ -829,9 +1017,9 @@ static int read_fbmodes_file (int w, int h)
       }
    } while (s);
 
-   free (mode_id);
-   free (geometry);
-   free (timings);
+   _AL_FREE (mode_id);
+   _AL_FREE (geometry);
+   _AL_FREE (timings);
 
    fclose (fbmodes);
    return ret;
@@ -898,6 +1086,7 @@ static void set_default_timings (void)
 static int update_timings(struct fb_var_screeninfo *mode)
 {
    struct timings *t;
+   ASSERT(mode);
 
    set_default_timings();
    t = _fb_get_timings (mode->xres, mode->yres);
@@ -952,8 +1141,15 @@ static void _fb_set_pixclock(int new_val)
  */
 void _module_init(int system_driver)
 {
-   if (system_driver == SYSTEM_LINUX)
+   if (system_driver == SYSTEM_LINUX) {
       _unix_register_gfx_driver(GFX_FBCON, &gfx_fbcon, TRUE, TRUE);
+      /* Register resolution and colour depth getters.
+       * This is a bit ugly because these are actually part of the
+       * system driver rather than the graphics driver...
+       */
+      system_linux.desktop_color_depth = __al_linux_get_fb_color_depth;
+      system_linux.get_desktop_resolution = __al_linux_get_fb_resolution;
+   }
 }
 
 #endif      /* ifdef ALLEGRO_MODULE */
