@@ -225,29 +225,29 @@ typedef struct AF_MODE_INFO      /* mode information structure */
    unsigned short BytesPerScanLine        __PACKED__;
    unsigned short BitsPerPixel            __PACKED__;
    unsigned short MaxBuffers              __PACKED__;
-   unsigned char  RedMaskSize             __PACKED__;
-   unsigned char  RedFieldPosition        __PACKED__;
-   unsigned char  GreenMaskSize           __PACKED__;
-   unsigned char  GreenFieldPosition      __PACKED__;
-   unsigned char  BlueMaskSize            __PACKED__;
-   unsigned char  BlueFieldPosition       __PACKED__;
-   unsigned char  RsvdMaskSize            __PACKED__;
-   unsigned char  RsvdFieldPosition       __PACKED__;
+   unsigned char  RedMaskSize;
+   unsigned char  RedFieldPosition;
+   unsigned char  GreenMaskSize;
+   unsigned char  GreenFieldPosition;
+   unsigned char  BlueMaskSize;
+   unsigned char  BlueFieldPosition;
+   unsigned char  RsvdMaskSize;
+   unsigned char  RsvdFieldPosition;
    unsigned short MaxBytesPerScanLine     __PACKED__;
    unsigned short MaxScanLineWidth        __PACKED__;
 
    /* VBE/AF 2.0 extensions */
    unsigned short LinBytesPerScanLine     __PACKED__;
-   unsigned char  BnkMaxBuffers           __PACKED__;
-   unsigned char  LinMaxBuffers           __PACKED__;
-   unsigned char  LinRedMaskSize          __PACKED__;
-   unsigned char  LinRedFieldPosition     __PACKED__;
-   unsigned char  LinGreenMaskSize        __PACKED__;
-   unsigned char  LinGreenFieldPosition   __PACKED__;
-   unsigned char  LinBlueMaskSize         __PACKED__;
-   unsigned char  LinBlueFieldPosition    __PACKED__;
-   unsigned char  LinRsvdMaskSize         __PACKED__;
-   unsigned char  LinRsvdFieldPosition    __PACKED__;
+   unsigned char  BnkMaxBuffers;
+   unsigned char  LinMaxBuffers;
+   unsigned char  LinRedMaskSize;
+   unsigned char  LinRedFieldPosition;
+   unsigned char  LinGreenMaskSize;
+   unsigned char  LinGreenFieldPosition;
+   unsigned char  LinBlueMaskSize;
+   unsigned char  LinBlueFieldPosition;
+   unsigned char  LinRsvdMaskSize;
+   unsigned char  LinRsvdFieldPosition;
    unsigned long  MaxPixelClock           __PACKED__;
    unsigned long  VideoCapabilities       __PACKED__;
    unsigned short VideoMinXScale          __PACKED__;
@@ -255,7 +255,7 @@ typedef struct AF_MODE_INFO      /* mode information structure */
    unsigned short VideoMaxXScale          __PACKED__;
    unsigned short VideoMaxYScale          __PACKED__;
 
-   unsigned char  reserved[76]            __PACKED__;
+   unsigned char  reserved[76];
 
 } AF_MODE_INFO;
 
@@ -268,11 +268,11 @@ typedef struct AF_MODE_INFO      /* mode information structure */
 typedef struct AF_DRIVER         /* VBE/AF driver structure */
 {
    /* driver header */
-   char           Signature[12]           __PACKED__;
+   char           Signature[12];
    unsigned long  Version                 __PACKED__;
    unsigned long  DriverRev               __PACKED__;
-   char           OemVendorName[80]       __PACKED__;
-   char           OemCopyright[80]        __PACKED__;
+   char           OemVendorName[80];
+   char           OemCopyright[80];
    unsigned short *AvailableModes         __PACKED__;
    unsigned long  TotalMemory             __PACKED__;
    unsigned long  Attributes              __PACKED__;
@@ -718,7 +718,7 @@ static int load_vbeaf_driver(AL_CONST char *filename)
    #else
 
       /* simple version for other platforms */
-      size = file_size(filename);
+      size = file_size_ex(filename);
       if (size <= 0)
 	 return 0;
 
@@ -728,10 +728,10 @@ static int load_vbeaf_driver(AL_CONST char *filename)
    if (!f)
       return 0;
 
-   af_driver = _accel_driver = malloc(size);
+   af_driver = _accel_driver = _AL_MALLOC(size);
 
    if (pack_fread(af_driver, size, f) != size) {
-      free(af_driver);
+      _AL_FREE(af_driver);
       af_driver = _accel_driver = NULL;
       return 0;
    }
@@ -1183,7 +1183,7 @@ static BITMAP *vbeaf_init(int w, int h, int v_w, int v_h, int color_depth)
     * The bank switchers assume asm-mode calling conventions, but the
     * library would try to call them with C calling conventions.
     */
-#ifdef ALLEGRO_USE_C
+#ifdef ALLEGRO_NO_ASM
    return NULL;
 #endif
 
@@ -1567,7 +1567,7 @@ static void vbeaf_exit(BITMAP *b)
    _remove_linear_mapping(&af_linear_mem);
 
    if (af_driver) {
-      free(af_driver);
+      _AL_FREE(af_driver);
       af_driver = _accel_driver = NULL;
    }
 
@@ -1645,18 +1645,18 @@ static GFX_MODE_LIST *vbeaf_fetch_mode_list(void)
    unsigned short *mode;
    int vbeaf_was_off;
 
-   mode_info = malloc(sizeof(AF_MODE_INFO));
+   mode_info = _AL_MALLOC(sizeof(AF_MODE_INFO));
    if (!mode_info) return NULL;
 
    /* make sure VBE/AF interface is enabled! */
    if (!af_driver) {
       if (!vbeaf_locate_driver()) {
-         if (mode_info) free(mode_info);
+         if (mode_info) _AL_FREE(mode_info);
 	 
          return NULL;
       }
       if (!vbeaf_lowlevel_init()) {
-         if (mode_info) free(mode_info);
+         if (mode_info) _AL_FREE(mode_info);
 	 
          return NULL;
       }
@@ -1666,9 +1666,9 @@ static GFX_MODE_LIST *vbeaf_fetch_mode_list(void)
       vbeaf_was_off = FALSE;
 
    /* start building mode-list */
-   gfx_mode_list = malloc(sizeof(GFX_MODE_LIST));
+   gfx_mode_list = _AL_MALLOC(sizeof(GFX_MODE_LIST));
    if (!gfx_mode_list) {
-      if (mode_info) free(mode_info);
+      if (mode_info) _AL_FREE(mode_info);
       
       return NULL;
    }
@@ -1679,15 +1679,15 @@ static GFX_MODE_LIST *vbeaf_fetch_mode_list(void)
    for (mode = af_driver->AvailableModes; *mode != 0xFFFF; mode++) {
       gfx_mode_list->mode = _al_sane_realloc(gfx_mode_list->mode, sizeof(GFX_MODE) * (gfx_mode_list->num_modes + 1));
       if (!gfx_mode_list->mode) {
-         if (mode_info) free(mode_info);
-         if (gfx_mode_list) free(gfx_mode_list);
+         if (mode_info) _AL_FREE(mode_info);
+         if (gfx_mode_list) _AL_FREE(gfx_mode_list);
          
          return NULL;
       }
       if (af_driver->GetVideoModeInfo(af_driver, *mode, mode_info) != 0) {
-         if (mode_info) free(mode_info);
-         if (gfx_mode_list->mode) free(gfx_mode_list->mode);
-         if (gfx_mode_list) free(gfx_mode_list);
+         if (mode_info) _AL_FREE(mode_info);
+         if (gfx_mode_list->mode) _AL_FREE(gfx_mode_list->mode);
+         if (gfx_mode_list) _AL_FREE(gfx_mode_list);
          
          return NULL;
       }
@@ -1702,8 +1702,8 @@ static GFX_MODE_LIST *vbeaf_fetch_mode_list(void)
    /* terminate mode list */
    gfx_mode_list->mode = _al_sane_realloc(gfx_mode_list->mode, sizeof(GFX_MODE) * (gfx_mode_list->num_modes + 1));
    if (!gfx_mode_list->mode) {
-      if (mode_info) free(mode_info);
-      if (gfx_mode_list) free(gfx_mode_list);
+      if (mode_info) _AL_FREE(mode_info);
+      if (gfx_mode_list) _AL_FREE(gfx_mode_list);
       
       return NULL;
    }
@@ -1716,7 +1716,7 @@ static GFX_MODE_LIST *vbeaf_fetch_mode_list(void)
    if (vbeaf_was_off)
       vbeaf_exit(NULL);
 
-   if (mode_info) free(mode_info);
+   if (mode_info) _AL_FREE(mode_info);
 
    return gfx_mode_list;
 }
