@@ -55,7 +55,7 @@ typedef struct MODE_LIST {
 typedef struct DRIVER_LIST {
    int       id;
    char      name[DRVNAME_SIZE];
-   void      *fetch_mode_list_ptr;
+   int       mode_list_owned;
    MODE_LIST *mode_list;
    int       mode_count;
 } DRIVER_LIST;
@@ -374,7 +374,7 @@ static int create_mode_list(DRIVER_LIST *driver_list_entry, FILTER_FUNCTION filt
       if (!filter) {
 	 driver_list_entry->mode_count = sizeof(default_mode_list) / sizeof(MODE_LIST) - 1;
 	 driver_list_entry->mode_list = default_mode_list;
-	 driver_list_entry->fetch_mode_list_ptr = NULL;  /* static */
+	 driver_list_entry->mode_list_owned = FALSE;
 	 return 0;
       }
 
@@ -403,7 +403,7 @@ static int create_mode_list(DRIVER_LIST *driver_list_entry, FILTER_FUNCTION filt
       }
 
       driver_list_entry->mode_list = temp_mode_list;
-      driver_list_entry->fetch_mode_list_ptr = (void *)1L;  /* private dynamic */
+      driver_list_entry->mode_list_owned = TRUE;
       return 0;
    }
 
@@ -429,6 +429,7 @@ static int create_mode_list(DRIVER_LIST *driver_list_entry, FILTER_FUNCTION filt
    }
 
    driver_list_entry->mode_list = temp_mode_list;
+   driver_list_entry->mode_list_owned = TRUE;
    destroy_gfx_mode_list(gfx_mode_list);
    return 0;
 }
@@ -443,7 +444,7 @@ static int create_driver_list(FILTER_FUNCTION filter)
 {
    _DRIVER_INFO *driver_info;
    GFX_DRIVER *gfx_driver;
-   int i, j, used_prefetched;
+   int i;
    int list_pos;
 
    if (system_driver->gfx_drivers)
@@ -451,7 +452,7 @@ static int create_driver_list(FILTER_FUNCTION filter)
    else
       driver_info = _gfx_driver_list;
 
-   driver_list = malloc(sizeof(DRIVER_LIST) * 3);
+   driver_list = _AL_MALLOC(sizeof(DRIVER_LIST) * 3);
    if (!driver_list) return -1;
 
    list_pos = 0;
@@ -480,25 +481,8 @@ static int create_driver_list(FILTER_FUNCTION filter)
       driver_list[list_pos].id = driver_info[i].id;
       gfx_driver = driver_info[i].driver;
       do_uconvert(gfx_driver->ascii_name, U_ASCII, driver_list[list_pos].name, U_CURRENT, DRVNAME_SIZE);
-      driver_list[list_pos].fetch_mode_list_ptr = gfx_driver->fetch_mode_list;
 
-      used_prefetched = FALSE;
-
-      /* use already fetched mode-list if possible */
-      for (j=0; j < list_pos; j++) {
-         if (driver_list[list_pos].fetch_mode_list_ptr == driver_list[j].fetch_mode_list_ptr) {
-            driver_list[list_pos].mode_list = driver_list[j].mode_list;
-            driver_list[list_pos].mode_count = driver_list[j].mode_count;
-            /* the following line prevents a mode-list from beeing free'd more than once */
-            driver_list[list_pos].fetch_mode_list_ptr = NULL;
-            used_prefetched = TRUE;
-            break;
-         }
-      }
-      /* didn't find an already fetched mode-list */
-      if (!used_prefetched) {
-         create_mode_list(&driver_list[list_pos], filter);
-      }
+      create_mode_list(&driver_list[list_pos], filter);
 
       if (driver_list[list_pos].mode_count > 0) {
 	 list_pos++;
@@ -524,11 +508,13 @@ static void destroy_driver_list(void)
    int driver;
 
    for (driver=0; driver < driver_count; driver++) {
-      if (driver_list[driver].fetch_mode_list_ptr)
-         free(driver_list[driver].mode_list);
+      if (driver_list[driver].mode_list_owned)
+         _AL_FREE(driver_list[driver].mode_list);
    }
 
-   free(driver_list);
+   _AL_FREE(driver_list);
+   driver_list = NULL;
+   driver_count = 0;
 }
 
 
@@ -625,14 +611,22 @@ static AL_CONST char *gfx_depth_getter(int index, int *list_size)
 /* gfx_mode_select_filter:
  *  Extended version of the graphics mode selection dialog, which allows the 
  *  user to select the color depth as well as the resolution and hardware 
- *  driver. This version of the function reads the initial values from the 
- *  parameters when it activates, so you can specify the default values.
- *  An optional filter can be passed to check whether a particular entry
- *  should be displayed or not.
+ *  driver. An optional filter can be passed to check whether a particular
+ *  entry should be displayed or not. Initial values for the selections may be
+ *  given at the addresses passed to the function, and the user's selection
+ *  will be stored at those addresses if the dialog is OK'd. Initial values
+ *  at the addresses passed to the function may be set to 0 or -1 to indicate
+ *  not to search for the values but to default to the first entries in each 
+ *  selection list.
+ *  In the case of an error, *card is set to GFX_NONE and FALSE is returned.
+ *  In the case that the filter filtered out all of the modes, *card is set to
+ *  GFX_NONE and TRUE is returned.
  */
 int gfx_mode_select_filter(int *card, int *w, int *h, int *color_depth, FILTER_FUNCTION filter)
 {
    int i, ret, what_driver, what_mode, what_bpp, extd;
+   MODE_LIST* mode_iter;
+   
    ASSERT(card);
    ASSERT(w);
    ASSERT(h);
@@ -649,45 +643,61 @@ int gfx_mode_select_filter(int *card, int *w, int *h, int *color_depth, FILTER_F
    what_dialog[GFX_OK].dp = (void*)get_config_text("OK");
    what_dialog[GFX_CANCEL].dp = (void*)get_config_text("Cancel");
 
-   create_driver_list(filter);
+   ret = create_driver_list(filter);
 
-   /* We try to use the values passed through the argument pointers
-    * as initial settings for the dialog boxes, but only if we have
-    * been called from the extended function.
+   if (ret == -1) {
+      *card = GFX_NONE;
+      return FALSE;
+   }
+
+   if (!driver_count) {
+      *card = GFX_NONE;
+      return TRUE;
+   }
+
+   /* The data stored at the addresses passed to this function is used to
+    * search for initial selections in the dialog lists. If the requested
+    * entries are not found, default to the first selection in each list in
+    * order of the driver, the mode w/h, and also the color depth when in
+    * extended mode (called directly or from gfx_mode_select_ex).
     */
-   if (extd) {
-      /* firstly the driver */
-      what_dialog[GFX_DRIVERLIST].d1 = 0;  /* GFX_AUTODETECT */
-
-      for (i=0; i<driver_count; i++) {
+   /* Check for the user suggested driver first */
+   what_driver = 0;/* Default to first entry if not found */
+   /* Don't search for initial values if *card is 0 or -1 */
+   if (!((*card == 0) || (*card == -1))) {
+      for (i = 0 ; i < driver_count ; ++i) {
          if (driver_list[i].id == *card) {
-            what_dialog[GFX_DRIVERLIST].d1 = i;
+            what_driver = i;
+         }
+      }
+   }
+   what_dialog[GFX_DRIVERLIST].d1 = what_driver;
+   what_dialog[GFX_CHANGEPROC].d1 = what_driver;
+   
+   /* Check for suggested resolution dimensions second */
+   what_mode = 0;/* Default to first entry if not found */
+   mode_iter = &(driver_list[what_driver].mode_list[0]);
+   /* Don't search for initial values if *w or *h is 0 or -1 */
+   if (!(((*w == 0) || (*w == -1)) || ((*h == 0) || (*h == -1)))) {
+      for (i = 0 ; i < driver_list[what_driver].mode_count ; ++i) {
+         if ((mode_iter->w == *w) && (mode_iter->h == *h)) {
+            what_mode = i;
             break;
          }
+         ++mode_iter;
       }
-
-      what_driver = what_dialog[GFX_DRIVERLIST].d1;
-      what_dialog[GFX_CHANGEPROC].d1 = what_dialog[GFX_DRIVERLIST].d1;
-
-      /* secondly the resolution */
-      what_dialog[GFX_MODELIST].d1 = 0;
-
-      for (i=0; driver_list[what_driver].mode_list[i].w; i++) {
-         if ((driver_list[what_driver].mode_list[i].w == *w)
-              && (driver_list[what_driver].mode_list[i].h == *h)) {
-            what_dialog[GFX_MODELIST].d1 = i;
-            break; 
-         }
+   }
+   what_dialog[GFX_MODELIST].d1 = what_mode;
+   what_dialog[GFX_CHANGEPROC].d2 = what_mode;
+   
+   /* Check for suggested color depth when in extended mode */
+   if (extd) {
+      what_bpp = 0;
+      /* Don't search for initial values if *color_depth is 0 or -1 */
+      if (!((*color_depth == 0) || (*color_depth == -1))) {
+         what_bpp = bpp_index_for_mode(*color_depth , what_driver , what_mode);
+         if (what_bpp < 0) {what_bpp = 0;} /* Default to first entry if not found */
       }
-
-      what_mode = what_dialog[GFX_MODELIST].d1;
-      what_dialog[GFX_CHANGEPROC].d2 = what_dialog[GFX_MODELIST].d1;  /* not d2 */
-
-      /* thirdly the color depth */
-      what_bpp = bpp_index_for_mode(*color_depth, what_driver, what_mode);
-      if (what_bpp < 0)
-         what_bpp = 0;
-
       what_dialog[GFX_DEPTHLIST].d1 = what_bpp;
    }
 
@@ -703,13 +713,14 @@ int gfx_mode_select_filter(int *card, int *w, int *h, int *color_depth, FILTER_F
    else
       what_bpp = 0;
 
-   *card = driver_list[what_driver].id;
-
-   *w = driver_list[what_driver].mode_list[what_mode].w;
-   *h = driver_list[what_driver].mode_list[what_mode].h;
-
-   if (extd)
-      *color_depth = bpp_value_for_mode(what_bpp, what_driver, what_mode);
+   if (ret != GFX_CANCEL) {
+      *card = driver_list[what_driver].id;
+      *w = driver_list[what_driver].mode_list[what_mode].w;
+      *h = driver_list[what_driver].mode_list[what_mode].h;
+      if (extd) {
+         *color_depth = bpp_value_for_mode(what_bpp, what_driver, what_mode);
+      }
+   }
 
    destroy_driver_list();
 
@@ -722,10 +733,12 @@ int gfx_mode_select_filter(int *card, int *w, int *h, int *color_depth, FILTER_F
 
 
 /* gfx_mode_select_ex:
- *  Extended version of the graphics mode selection dialog, which allows the 
- *  user to select the color depth as well as the resolution and hardware 
- *  driver. This version of the function reads the initial values from the 
- *  parameters when it activates, so you can specify the default values.
+ *  Extended version of the graphics mode selection dialog, which allows the
+ *  user to select the color depth as well as the resolution and hardware
+ *  driver. Initial values for the selections will be looked for at the
+ *  addresses passed to the function, and selections will be stored there if
+ *  the user does not cancel the dialog. See gfx_mode_select_filter for
+ *  details and return values.
  */
 int gfx_mode_select_ex(int *card, int *w, int *h, int *color_depth)
 {
@@ -740,22 +753,17 @@ int gfx_mode_select_ex(int *card, int *w, int *h, int *color_depth)
 
 /* gfx_mode_select:
  *  Displays the Allegro graphics mode selection dialog, which allows the
- *  user to select a screen mode and graphics card. Stores the selection
- *  in the three variables, and returns zero if it was closed with the 
- *  Cancel button, or non-zero if it was OK'd.
+ *  user to select a screen mode and graphics card. Initial values for the
+ *  selection will be looked for at the addresses passed to the function, and
+ *  the selection will be stored in the three variables if the dialog is not
+ *  cancelled. See gfx_mode_select_filter for details and return values.
  */
 int gfx_mode_select(int *card, int *w, int *h)
 {
    ASSERT(card);
    ASSERT(w);
    ASSERT(h);
-
-   /* Make sure these values are not used uninitialised.
-    * This is different to the other gfx_mode_select_* functions.
-    */
-   *card = GFX_AUTODETECT;
-   *w = 0;
-   *h = 0;
-
    return gfx_mode_select_filter(card, w, h, NULL, NULL);
 }
+
+
