@@ -70,11 +70,9 @@ int waveform_proc(int msg, DIALOG *d, int c)
 	 min = MAX(d->x+MIN(i,j)-6, d->x+1);                \
 	 max = MIN(d->x+MAX(i,j)+6, d->x+d->w-2);           \
 	 oldval = newval;                                   \
-	 show_mouse(NULL);                                  \
-	 set_clip(screen, min, d->y+1, max, d->y+d->h-2);   \
+	 set_clip_rect(screen, min, d->y+1, max, d->y+d->h-2); \
 	 object_message(d, MSG_DRAW, 1);                    \
-	 set_clip(screen, 0, 0, SCREEN_W-1, SCREEN_H-1);    \
-	 show_mouse(screen);                                \
+	 set_clip_rect(screen, 0, 0, SCREEN_W-1, SCREEN_H-1);  \
       }                                                     \
    }
 
@@ -162,7 +160,7 @@ int waveform_proc(int msg, DIALOG *d, int c)
 
 	    if (s) {
 	       if (!c)
-		  set_clip(screen, d->x+1, d->y+1, d->x+d->w-2, d->y+d->h-2);
+		  set_clip_rect(screen, d->x+1, d->y+1, d->x+d->w-2, d->y+d->h-2);
 
 	       val = SAMPLE_TO_SCREEN(s->loop_start);
 	       if ((val > 0) && (val < d->w)) {
@@ -187,7 +185,7 @@ int waveform_proc(int msg, DIALOG *d, int c)
 		  vline(screen, d->x+val, d->y+1, d->y+d->h-2, palette_color[4]);
 
 	       if (!c)
-		  set_clip(screen, 0, 0, SCREEN_W-1, SCREEN_H-1);
+		  set_clip_rect(screen, 0, 0, SCREEN_W-1, SCREEN_H-1);
 	    }
 	 }
 	 else {
@@ -237,23 +235,23 @@ int waveform_proc(int msg, DIALOG *d, int c)
 	       val = atoi(thedialog[VOLUME].dp);
 	       i = atoi(thedialog[END_VOLUME].dp);
 	       j = atoi(thedialog[VOLUME_TIME].dp);
-	       voice_set_volume(d->d1, MID(0, val, 255));
+	       voice_set_volume(d->d1, CLAMP(0, val, 255));
 	       if (j > 0)
-		  voice_ramp_volume(d->d1, MID(0, j, 999999), MID(0, i, 255));
+		  voice_ramp_volume(d->d1, CLAMP(0, j, 999999), CLAMP(0, i, 255));
 
 	       val = atoi(thedialog[FREQ].dp);
 	       i = atoi(thedialog[END_FREQ].dp);
 	       j = atoi(thedialog[FREQ_TIME].dp);
-	       voice_set_frequency(d->d1, MID(1000, val, 99999));
+	       voice_set_frequency(d->d1, CLAMP(1000, val, 99999));
 	       if (j > 0)
-		  voice_sweep_frequency(d->d1, MID(0, j, 999999), MID(1000, i, 99999));
+		  voice_sweep_frequency(d->d1, CLAMP(0, j, 999999), CLAMP(1000, i, 99999));
 
 	       val = atoi(thedialog[PAN].dp);
 	       i = atoi(thedialog[END_PAN].dp);
 	       j = atoi(thedialog[PAN_TIME].dp);
-	       voice_set_pan(d->d1, MID(0, val, 255));
+	       voice_set_pan(d->d1, CLAMP(0, val, 255));
 	       if (j > 0)
-		  voice_sweep_pan(d->d1, MID(0, j, 999999), MID(0, i, 255));
+		  voice_sweep_pan(d->d1, CLAMP(0, j, 999999), CLAMP(0, i, 255));
 
 	       voice_start(d->d1); 
 	    }
@@ -426,15 +424,18 @@ int main(int argc, char *argv[])
    if (argc > 1)
       strcpy(samplename, argv[1]);
 
-   allegro_init();
+   if (allegro_init() != 0)
+      return 1;
    install_keyboard();
    install_mouse();
    install_timer();
 
-   if (set_gfx_mode(GFX_SAFE, 640, 480, 0, 0) != 0) {
-      set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
-      allegro_message("Error setting graphics mode\n%s\n", allegro_error);
-      return 1;
+   if (set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0) != 0) {
+      if (set_gfx_mode(GFX_SAFE, 640, 480, 0, 0) != 0) {
+	 set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
+	 allegro_message("Error setting graphics mode\n%s\n", allegro_error);
+	 return 1;
+      }
    }
 
    /* we use only one voice */
@@ -469,4 +470,4 @@ int main(int argc, char *argv[])
    return 0;
 }
 
-END_OF_MAIN();
+END_OF_MAIN()
